#!/usr/bin/ksh
#
# sv_lib.ksh -- Services Tier Library Functions
#
# This script library is designed to work ONLY under the ESM common
# scripting system.
#
# All required shell libraries are expected to be loaded in advance
# of invocation of any functions defined herein.
#
# Copyright 2003 Sun Microsystems, Inc. All Rights Reserved.
#
###

# --------------------------------------------------------------------
# Post Install Function
#
sv_postinstall () {

    # Create requisite records in the database for identification
    # and subscriptions for various Service Tier subcomponents.
    #
    el_log 1 "Loading Service Tier Factory Data..."

    _sv_perform_factory_load=true

    if el_hook_is_defined use_existing_db ; then
        __prompt="Use your existing database on `el_configvar_get ORACLE_DB_SERVER`"
        if el_isEsmTrue `ckyorn -Q -d n -p "$__prompt"` ; then
            el_log 1 "[Using existing database...no Service Tier factory data load will be performed]"
            _sv_perform_factory_load=false
        fi
    fi

    if $_sv_perform_factory_load ; then
        # Fetch the configvars needed for the factory load
        #
        for varname in ORACLE_DB_SERVER ORACLE_DB_PORT ORACLE_DB_SID ORACLE_DB_LOGIN_ID ORACLE_DB_PASSWORD
        do
            eval $varname="`el_configvar_get $varname`"
            if [ -z "$varname" ] ; then
               el_assert "Configuration variable '$varname' is not defined!"
            fi
        done

        CP=$ESM_LIB/esm-util.jar
        CP=$CP:$ESM_LIB/esm-data.jar
        CP=$CP:$ESM_PLATFORM_LIB/esm-services.jar
        CP=$CP:$ESM_PLATFORM_LIB/esm-scanners.jar
        CP=$CP:$ESM_PLATFORM_LIB/esm-agent.jar
        CP=$CP:$ESM_LIB/esm-shared.jar
        CP=$CP:$ESM_LIB/ojdbc14.jar
        CP=$CP:$ESM_LIB/xsu12.jar
        CP=$CP:$ESM_LIB/wbem.jar

        $JAVA -cp "$CP" -Desm.config=$ESM_CONFIG com.sun.netstorage.mgmt.service.factoryload.FactoryLoad \
              -server $ORACLE_DB_SERVER \
              -port $ORACLE_DB_PORT     \
              -sid $ORACLE_DB_SID       \
              -user $ORACLE_DB_LOGIN_ID \
              -pass $ORACLE_DB_PASSWORD \
              >/dev/null
        if [ $? -ne 0 ]; then
            el_assert "Failure to load Service Tier factory data"
        fi
    fi

    # Generate and store the CA and client keys needed for SSL
    #
    el_log 1 ""
    el_log 1 "Generating the ESM Certificate Authority (CA) keys and certificates..."

    CR_ENCRYPTION_KEY=`el_configvar_get CR_ENCRYPTION_KEY`

    KEYS_DIRECTORY="`grep com.sun.netstorage.mgmt.keysDirectory= $ESM_CONFIG/esm.properties | head -1 | cut -d'=' -f2`"
    CA_KEYSTORE_NAME="`grep com.sun.netstorage.mgmt.ca.keyStore= $ESM_CONFIG/esm.properties | head -1 | cut -d'=' -f2`"
    CA_KEYSTORE_PWD="$CR_ENCRYPTION_KEY"
    CA_TRUSTSTORE_NAME="`grep com.sun.netstorage.mgmt.ca.trustStore= $ESM_CONFIG/esm.properties | head -1 | cut -d'=' -f2`"

    $ESM_PLATFORM_BIN/cagenkey.sh "$CA_KEYSTORE_NAME" "$CA_KEYSTORE_PWD" "$CA_TRUSTSTORE_NAME" "$KEYS_DIRECTORY" ||
        el_assert "Failure to generate CA key and certificate."

    el_log 1 "Generating the Presentation Tier keys and certificates..."

    PT_KEY_ALIAS="`grep com.sun.netstorage.mgmt.pt.keyAlias= $ESM_CONFIG/esm.properties | head -1 | cut -d'=' -f2`"
    PT_COMMON_NAME="`grep com.sun.netstorage.mgmt.pt.keyCommonName= $ESM_CONFIG/esm.properties | head -1 | cut -d'=' -f2`"
    PT_KEYSTORE_NAME="`grep com.sun.netstorage.mgmt.pt.keyStore= $ESM_CONFIG/esm.properties | head -1 | cut -d'=' -f2`"
    PT_KEYSTORE_PWD="$CR_ENCRYPTION_KEY"
    PT_TRUSTSTORE_NAME="`grep com.sun.netstorage.mgmt.pt.trustStore= $ESM_CONFIG/esm.properties | head -1 | cut -d'=' -f2`"

    $ESM_PLATFORM_BIN/keygenandsign.sh "$PT_KEYSTORE_NAME"   "$PT_KEYSTORE_PWD" \
                                       "$PT_TRUSTSTORE_NAME" "$PT_KEY_ALIAS"    \
                                       "$PT_COMMON_NAME"     "$KEYS_DIRECTORY"  ||
        el_assert "Failure to generate Presentation Tier client keys and certificates."


    el_log 1 "Setting security information in the smcwebserver..."

    LOCKHART_PASSWORD_KEY="`grep LOCKHART_PASSWORD_KEY= $ESM_CONFIG/esm.properties | cut -d'=' -f2`"

    addSMICCkey "$LOCKHART_PASSWORD_KEY" "$PT_KEYSTORE_PWD" ||
        el_assert "Failure to set the Presentation Tier keystore password in the smcwebserver."

    addSMICCkey ORACLE_DB_LOGIN_ID "`el_configvar_get ORACLE_DB_LOGIN_ID`" ||
        el_assert "Failure to set the Oracle user login ID in the smcwebserver."

    addSMICCkey ORACLE_DB_PASSWORD "`el_configvar_get ORACLE_DB_PASSWORD`" ||
        el_assert "Failure to set the Oracle user password in the smcwebserver."


    # Create the ESM reports directory tree
    #
    _esmreportsroot=`el_configvar_get esm.rdp.report.report_dir`

    el_log 1 "Creating ESM reports directory tree..."

    /usr/bin/mkdir -p -m 755 $_esmreportsroot/failed   &&
    /usr/bin/mkdir -p -m 755 $_esmreportsroot/finished ||
       el_assert "Failure to create the ESM reports directory tree"

    return 0
}


# --------------------------------------------------------------------
# Preremove Function
#
sv_preremove () {

    # Remove the reports directory tree, but IFF the user has NOT
    # indicated a "full" uninstall (ie, NOT "esmuninstall -f");
    # that is, save the reports directory tree if a simple "esmuninstall"
    # has been specified.
    #
    el_isEsmTrue "$ESM_PERSIST"
    if [ $? -ne 0 ] ; then
        __reptree=`el_configvar_get esm.rdp.report.report_dir`
        if [ -d "$__reptree" ] ; then
            el_log 1 "Removing ESM reports subdirectories..."
            /usr/bin/rm -rf "$__reptree/failed" "$__reptree/finished"
	    [ -d "$__reptree/failed" -o -d "$__reptree/finished" ] ||
                el_slog 1 "WARNING: unable to remove all $__reptree subdirectories!"
        fi
    fi

    # Remove all security keys created during the postinstall
    #
    el_log 1 "Removing security information in the smcwebserver..."

    /usr/bin/rm -rf $ESM_CONFIG/keys

    LOCKHART_PASSWORD_KEY="`grep LOCKHART_PASSWORD_KEY= $ESM_CONFIG/esm.properties | cut -d'=' -f2`"

    removeSMICCkey "$LOCKHART_PASSWORD_KEY" ||
        el_log 1 "WARNING: unable to remove the Presentation Tier keystore password in the smcwebserver"

    removeSMICCkey "ORACLE_DB_LOGIN_ID" ||
        el_log 1 "WARNING: unable to remove the Oracle user login ID in the smcwebserver"

    removeSMICCkey "ORACLE_DB_PASSWORD" ||
        el_log 1 "WARNING: unable to removethe Oracle user password in the smcwebserver"

    return 0
}


# --------------------------------------------------------------------
# Component start Function
#
sv_start () {

    # If the Services container is already running, then display an
    # info message and return with success; otherwise, continue to
    # startup the container.
    #
    # A two-level check is done here: test interaction with the JobManager,
    # then if that fails, see if the process exists (in which case, kill it).
    #
    if $ESM_BASE/platform/container/bin/pingcrcre > /dev/null 2>&1 ; then
	el_slog 1 "[CR Service Tier container is already running]"
	return 0
    fi

    if pf_isProcessRunning crcre > /dev/null ; then
	pid=`pf_getPid crcre`
	el_slog 1 "[CR Service Tier container appears to be hung (pid: $pid)...restarting it]"
	if pf_killProcess crcre ; then
	    el_slog 1 "[Hung Services container has been killed]"
	    pf_deletePidFile crcre
	else
	    el_slog 1 "WARNING: Failure to kill the hung Services container process (pid: $pid)"
	fi
    fi

    # Set the ulimit coredump value to "unlimited" to ensure potential core files
    # can be created without disk space restrictions.
    #
    ulimit -c unlimited ||
	el_log 1 "WARNING: unable to set crcre ulimit coredump value to 'unlimited'"

    # Change working directory to the prescribed directory,
    # then preserve the core file, if it exists.
    #
    CONTAINER_RUNDIR=$ESM_VARBASE/crcre/run

    if cd $CONTAINER_RUNDIR ; then
        if [ -f core ] ; then
            /usr/bin/mv core core.old ||
                el_log 1 "WARNING: unable to preserve existing core file in $CONTAINER_RUNDIR"
        fi
    else
	el_log 1 "WARNING: unable to change working directory to $CONTAINER_RUNDIR"
    fi

    # Startup the Service Tier container using the standard script
    #
    if /usr/bin/setpgrp $ESM_PLATFORM/container/bin/startcre > $ESM_VARBASE/log/cr.container.out 2>&1
    then
        el_slog 1 "[CR Service Tier container started]"
    else
        el_slog 1 "Failure to start the CR Service Tier container"
        return 1
    fi

    return 0
}


# --------------------------------------------------------------------
# Component stop Function
#
sv_stop () {

    # NOTE: The creation of the Services container pid file is handled
    #       by the "startcre" script (see the sv_start function). There
    #       is no symmetric "stopcre" script--everything needed for
    #       proper Services container shutdown is performed here.
    #
    if pf_isProcessRunning crcre > /dev/null ; then
	if pf_killProcess crcre ; then
	    pf_deletePidFile crcre
	else
	    pid=`pf_getPid crcre`
	    el_slog 1 "WARNING: Failure to kill the hung CR Service Tier container process (pid: $pid)"
	fi
    fi

    return 0
}


# --------------------------------------------------------------------
# Component watch Function
#
# This function must run a lot quieter than most others due to the fact
# that it is designed to run under the cron-oriented 'esmwatch' environment.
#
sv_watch () {

    _svrc=0

    svtmpfile=$ESM_TMP/sv_watch.tmp

    $ESM_BASE/platform/container/bin/pingcrcre > $svtmpfile 2>&1
    if [ $? -ne 0 ] ; then
	el_slog 1 "`/usr/bin/cat $svtmpfile`"
	el_slog 1 "CR Service Tier container not responding...restarting it now"

	sv_start noping > $svtmpfile 2>&1
	_svrc=$?
	if [ $_svrc -ne 0 ] ; then
	    el_slog 1 "`/usr/bin/cat $svtmpfile`"
	    el_slog 1 "Unable to restart the CR Service Tier container"
	fi
    fi

    /usr/bin/rm -f $svtmpfile

    return $_svrc
}
